#' Compute EI criterion
#'
#' @param thetaN a proposed value for theta
#' @param krig list of lists of krig for different seasons
#' @param logy list of observed biomass for different seasons
#' @param sigma2 variance of the observational noise
#' @param vmincourant vector of current minimal value of squared differences. Its size is the total number of observations.
#' @param nsim number of simulations used to compute the EI
#'
#' @return EI value
#' @export
#'
#' @examples
#' ## Kriging approximation
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met0809 <- BuildMeteodat("2008-2009")
#' met0910 <- BuildMeteodat("2009-2010")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0809,Dun[i,])[times]))
#' y = log(y)
#' krig1 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0910,Dun[i,])[times]))
#' y = log(y)
#' krig2 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' krig = list(krig1,krig2)
#'  ## Simulation of observation data
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' nT=3
#' times = seq(160,305,length=nT)
#' listlogy = list(fbiomass(200,met0809,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),fbiomass(200,met0910,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)))
#' computeEI(rep(.5,3),krig,listlogy,.1,c(50,50))
computeEI = function(thetaN, krig, logy, sigma2, vmincourant, nsim = 200){

  # compute krig prediction for this thetaN
  preds = lapply(1:length(krig),function(k){
    predkrig= predictmultdim(thetaN,krig[[k]])
  })

  matpreds = Reduce(cbind,preds)
  n_outs = ncol(matpreds)

  veclogy = Reduce(c,logy)
  matlogy = matrix(veclogy,nrow=nsim,ncol=n_outs,byrow=TRUE)

  # simulate kriging realisation
  simWal = matrix(rnorm(n_outs*nsim, matpreds[1,], matpreds[2,]), nsim, n_outs, byrow = TRUE)

  # compute improvement
  Improvement = matrix(vmincourant,nsim,n_outs,byrow=TRUE)-(simWal-matlogy)^2/sigma2
  Improvement[Improvement<0] = 0
  EI = sum(colMeans(Improvement))
  EI
}


#' Find new point to run FSPM
#'
#' @param npoints design points to try
#' @param mincourant current minimal value of squared differences
#' @inheritParams computeEI
#'
#' @return list
#' @export
#'
#' @examples
#' ## Kriging approximation
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met0809 <- BuildMeteodat("2008-2009")
#' met0910 <- BuildMeteodat("2009-2010")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0809,Dun[i,])[times]))
#' y = log(y)
#' krig1 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0910,Dun[i,])[times]))
#' y = log(y)
#' krig2 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' krig = list(krig1,krig2)
#'  ## Simulation of observation data
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' nT=3
#' times = seq(160,305,length=nT)
#' listlogy = list(fbiomass(200,met0809,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),fbiomass(200,met0910,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)))
#' Findnewcall(10,krig,listlogy,.1,c(50,50))
Findnewcall = function(npoints = 100, krig, logy, sigma2, mincourant, nsim = 200){

  X = krig[[1]][[1]]@X
  dimension = ncol(X)
  n = npoints

  cpt=0
  while (cpt<4){
    Dprop = DiceDesign::lhsDesign(n, dimension)$design
    # add points around 5 interesting values
    ss = rowSums(computeSS(krig,logy,sigma2))
    ind = order(ss)[1:5]
    for (k in ind)
    {
      newpoints = matrix(as.vector(X[k,]),nrow= npoints,ncol=dimension,byrow=TRUE) + rnorm(dimension*npoints,0,.05)
      Dprop = rbind(Dprop,newpoints)
    }

    # compute EI for all proposed points
    if (!Sys.info()["sysname"]=="Windows"){
      nbCoresToUse <- parallel::detectCores() - 1
      vEI = bettermc::mclapply(1:nrow(Dprop), function(i){
        res=computeEI( Dprop[i,], krig, logy, sigma2, mincourant, nsim)
        return(res)
      }, mc.cores = nbCoresToUse)} else {
        vEI = lapply(1:nrow(Dprop), function(i){
          res=computeEI( Dprop[i,], krig, logy, sigma2, mincourant, nsim)
          return(res)
        })}

    vEI = Reduce(c, vEI)
    cpt = cpt+1
    if (max(vEI)>0) cpt=10 else (print("Findnewcall redone"))
  }

  best = which.max(vEI)
  thetanew = Dprop[best,]
  return(list(thetanew = thetanew, critEI = max(vEI)))
}



#' Make a new call to the FSPM and stack it to the already made
#'
#' @param yX a list of outputs of the FSPM
#' @param thetanew a new vector of parameter where the FSPM has to be run
#' @param seasons a list with the names of the considered seasons
#' @param x sowing density
#' @param times times of observation
#' @param Bounds a length(x)*2 matrix with lower and upper bounds for each dimension of x
#'
#' @return an updated list of outputs of the FSPM at the Kriging design points. The length of the list is the number of considered season.
#' @export
#'
#' @examples
#'  ## Kriging approximation
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met0809 <- BuildMeteodat("2008-2009")
#' met0910 <- BuildMeteodat("2009-2010")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0809,Dun[i,])[times]))
#' y = log(y)
#' krig1 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0910,Dun[i,])[times]))
#' y = log(y)
#' krig2 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' krig = list(krig1,krig2)
#' ## new simu to add
#' yX = lapply(krig,function(krigseason){
#' temp=sapply(1:length(krigseason),function(k){krigseason[[k]]@y})
#' temp
#' })
#' addsimu(yX,rep(.5,3),list("2008-2009","2009-2010"),200,times,Bounds)
addsimu = function(yX,thetanew,seasons,x,times,Bounds)
{
  thetanew = unNormalise(thetanew,Bounds)

  yNEW=lapply(seasons, function(season){
    datameteo = BuildMeteodat(season)
    y = fbiomass(x,datameteo,thetanew)[times]
    y = log(y)
    y[is.infinite(y)] = -10
    return(y)})

  yXreturn = lapply(1:length(yX), function(k) rbind(yX[[k]],yNEW[[k]]))

  yXreturn
}


#' Enhance the Kriging metamodel for calibration according to the EGO criterion
#'
#' @inheritParams computeEI
#' @param x sowing density
#' @param seasons a list with the names of the considered seasons
#' @param Bounds length(x)*2 matrix with lower and upper bounds for each dimension of x
#' @param times times of observations
#' @param nEI number of points to add
#' @param npoints design points to try
#'
#' @return a list with the updated krig metamodel, the sum of the minimal values, the value of the EI criterion for the added points
#' @export
#'
#' @examples
#' #'  ## Kriging approximation
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met0809 <- BuildMeteodat("2008-2009")
#' met0910 <- BuildMeteodat("2009-2010")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0809,Dun[i,])[times]))
#' y = log(y)
#' krig1 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0910,Dun[i,])[times]))
#' y = log(y)
#' krig2 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' krig = list(krig1,krig2)
#' ## new simu to add
#' yX = lapply(krig,function(krigseason){
#' temp=sapply(1:length(krigseason),function(k){krigseason[[k]]@y})
#' temp
#' })
#' listlogy = list(fbiomass(200,met0809,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),fbiomass(200,met0910,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)))
#' updatedKrig = runEGO(krig,listlogy,.1,200,list("2008-2009","2009-2010"),Bounds,times,1,10,50)
runEGO = function(krig,logy,sigma2,x,seasons,Bounds,times,nEI=5,npoints=100,nsim=100)
{
  X = krig[[1]][[1]]@X
  ssmat = computeSS(krig,logy,sigma2)
  vmincourant = apply(ssmat,2,min)
  n = nrow(X)
  yX = lapply(krig,function(krigseason){
    temp=sapply(1:length(krigseason),function(k){krigseason[[k]]@y})
    temp
  })

  valEI = numeric(nEI)
  valmin = numeric(nEI+1)
  valmin[1]=sum(vmincourant)

  for (j in 1:nEI)
  {
    # find a good theta
    newtheta=Findnewcall(npoints,krig,logy,sigma2,vmincourant,nsim=nsim)
    print(newtheta)
    valEI[j] = newtheta$critEI

    # run simulation
    X = rbind(X,newtheta$thetanew)
    ### adding output of the model
    yX = addsimu(yX,newtheta$thetanew,seasons,x,times,Bounds)

    # redo kriging
    krig = lapply(yX,function(y)
    {
      lapply(1:ncol(y), function(k){
        m <- DiceKriging::km(design=X, response=y[,k],control=list(trace=FALSE))
        return(m)
      })
    })


    # update the current min
    ssmat = computeSS(krig,logy,sigma2)
    vmincourant = apply(ssmat,2,min)
    valmin[j+1]= sum(vmincourant)
    print(sum(vmincourant))
  }

  return(list(krig=krig,valmin=valmin,vEI=valEI))
}
