
#' Compute log of the posterior distribution
#'
#' @param thetaN vector of parameters to calibrate
#' @param logy log of biomass observations
#' @param x sowing density
#' @param datmeteo meteorological data
#' @param times times of biomass observation
#' @param sigma2 variance of the observational noise
#' @param Bounds a length(x)*2 matrix with lower and upper bounds for each dimension of x
#'
#' @return value of the log posterior distribution
#' @export
#'
#' @examples
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' nT=3
#' times = seq(160,305,length=nT)
#' met = BuildMeteodat("2008-2009")
#' exactlogpost(Normalise(c(800,1600,3),Bounds),fbiomass(200,met,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),200,met,times,sigma2=.1,Bounds)
exactlogpost = function(thetaN,logy,x,datmeteo,times,sigma2=.1^2,Bounds)
{

  # prior distribution on normalised thetas, uniform on [0,1]^d
  if (any(thetaN<0) | any(thetaN>1)) return(-Inf)

  # unnormalised theta
  thetaUN = unNormalise(thetaN,Bounds)

  res=-.5 * (length(times)*log(sigma2) + sum((logy-log(fbiomass(x,datmeteo,thetaUN)[times]))^2)/sigma2)
  res
}



#' Compute log posterior distribution for several datasets observed during different seasons
#'
#' @inheritParams exactlogpost
#' @param listdatmeteo list of meteorological data from the seasons
#' @param listlogy list of observed biomass for different seasons
#'
#' @return value of the log posterior distribution
#' @export
#'
#' @examples
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' nT=3
#' times = seq(160,305,length=nT)
#' met0809 = BuildMeteodat("2008-2009")
#' met0910 = BuildMeteodat("2009-2010")
#' listmet = list(met0809,met0910)
#' listy = list(fbiomass(200,met0809,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),fbiomass(200,met0910,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)))
#' exactlogpostMulti(Normalise(c(800,1600,3),Bounds),listy,200,listmet,times,sigma2=.1,Bounds)
exactlogpostMulti = function(thetaN,listlogy,x,listdatmeteo,times,sigma2,Bounds)
{
  vrais= sapply(1:length(listlogy),function(k)
  {exactlogpost(thetaN,listlogy[[k]],x,listdatmeteo[[k]],times,sigma2,Bounds)}
  )
  sum(vrais)
}


#' Compute Kriging metamodel prediction
#'
#' @param theta vector of parameters
#' @param krig1s list of km object for Kriging metamodels for the same season and different observation time
#'
#' @return a matrix with two rows, first row contains the predictions, second row contains the sd of the uncertainties on the prediction
#' @export
#'
#' @examples
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met <- BuildMeteodat("2008-2009")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met,Dun[i,])[times]))
#' y = log(y)
#' krig1s = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' predictmultdim(rep(.5,3),krig1s)
predictmultdim = function(theta,krig1s)
{

  namesX = colnames(krig1s[[1]]@X)
  machin=sapply(krig1s,function(m)
  {
    new = (matrix(theta,nrow=1))
    colnames(new)= namesX
    res = predict(m,newdata=new,type="UK")
    c(res$mean,res$sd)
  })
  return(machin)

}


#' Compute log posterior distribution via a Kriging Metamodel approximation
#'
#' @param thetaN vector of parameters to calibrate
#' @param logy log of biomass observations
#' @param krig1s list of km object for Kriging metamodels for the same season and different observation time
#' @param sigma2 variance of the observational noise
#'
#' @return value of the log posterior distribution
#' @export
#'
#' @examples
#' ## Kriging approximation
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met <- BuildMeteodat("2008-2009")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met,Dun[i,])[times]))
#' y = log(y)
#' krig1s = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' ## Computation of the log posterior distribution
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' nT=3
#' times = seq(160,305,length=nT)
#' approxlogpost(Normalise(c(800,1600,3),Bounds),fbiomass(200,met,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),krig1s,sigma2=.1)
approxlogpost = function(thetaN,logy,krig1s,sigma2=.1^2)
{
  if (any(thetaN<0) | any(thetaN>1)) return(-Inf)

  predkrig= predictmultdim(thetaN,krig1s)

  res=-.5 *sum( log(sigma2+predkrig[2,]^2) + (logy - predkrig[1,] )^2/(sigma2+predkrig[2,]^2) )
  res
}



#' Compute log posterior distribution via a Kriging Metamodel approximation for several datasets observed during different seasons
#'
#' @inheritParams exactlogpost
#' @param listlogy list of observed biomass for different seasons
#' @param krig list of lists of krig for different seasons
#'
#' @return value of the log posterior distribution
#' @export
#'
#' @examples
#' ## Kriging approximation
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met0809 <- BuildMeteodat("2008-2009")
#' met0910 <- BuildMeteodat("2009-2010")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0809,Dun[i,])[times]))
#' y = log(y)
#' krig1 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0910,Dun[i,])[times]))
#' y = log(y)
#' krig2 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' krig = list(krig1,krig2)
#'  ## Computation of the log posterior distribution
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' nT=3
#' times = seq(160,305,length=nT)
#' listlogy = list(fbiomass(200,met0809,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),fbiomass(200,met0910,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)))
#' approxlogpostMulti(Normalise(c(800,1600,3),Bounds),listlogy,krig,sigma2=.1)
approxlogpostMulti = function(thetaN,listlogy,krig,sigma2)
{
  vrais= sapply(1:length(listlogy),function(k)
  {approxlogpost(thetaN,listlogy[[k]],krig[[k]],sigma2)}
  )
  sum(vrais)
}


#' Compute the squared differences between observations and outputs of the FSPM
#'
#' @param krig list of lists of krig for different seasons
#' @param logy list of observed biomass for different seasons
#' @param sigma2 variance of the observational noise
#'
#' @return a matrix with n rows and d columns of the squared differences between the d observed biomass for the n calls made to build the Kriging design
#' @export
#'
#' @examples
#' ## Kriging approximation
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met0809 <- BuildMeteodat("2008-2009")
#' met0910 <- BuildMeteodat("2009-2010")
#' nT=3
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0809,Dun[i,])[times]))
#' y = log(y)
#' krig1 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met0910,Dun[i,])[times]))
#' y = log(y)
#' krig2 = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .1,control=list(trace=FALSE))
#' return(m)})
#' krig = list(krig1,krig2)
#'  ## Computation of the log posterior distribution
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' nT=3
#' times = seq(160,305,length=nT)
#' listlogy = list(fbiomass(200,met0809,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)),fbiomass(200,met0910,c(800,1600,3))[times]+rnorm(nT,0,sqrt(.1)))
#' computeSS(krig,listlogy,.1)
computeSS = function(krig,logy,sigma2)
{
  # retrieve kriging design
  X = krig[[1]][[1]]@X
  n=nrow(X)

  # retrieve output of the FSPM used for fitting Kriging metamodels
  yX = lapply(krig,function(krigseason){
    temp=sapply(1:length(krigseason),function(k){krigseason[[k]]@y})
    temp
  })

  # compute the squared differences
  ss = lapply(1:length(logy),function(k){
    ((yX[[k]] - matrix(logy[[k]],nrow(X),ncol(yX[[k]]),byrow=TRUE))^2)/sigma2
  })

  ss = Reduce(cbind,ss)
  ss
}
