#' Probabilistic prediction from a distribution of theta
#'
#' @inheritParams fbiomass
#' @param distribtheta sample of parameters in the unit cube
#' @param Bounds lower and upper bounds for parameters
#'
#' @return prediction for the biomass
#' @export
#'
#' @examples
#' met <- BuildMeteodat("2008-2009")
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' Predbiomass = predictExact(200,met,matrix(runif(300,.2,.8),100,3),Bounds)
#' meanBiom = apply(Predbiomass,2,mean)
#' q5 = apply(Predbiomass,2,quantile,.05) ; q95 = apply(Predbiomass,2,quantile,.95)
#' plot(met$cumdegree,meanBiom,type="l",ylim=c(0,14))
#'  lines(met$cumdegree,q5,lty=2) ; lines(met$cumdegree,q95,lty=2)
#'  legend("topleft",legend = c("0.05 quantile","mean","0.95 quantile"),lty=c(2,1,2))
predictExact =  function(x, datmeteo, distribtheta,Bounds)
{


  distribthetaUN = unNormalise(distribtheta,Bounds)

  ntheta=nrow(distribtheta)
  preds = sapply(1:ntheta,function(k)
  {
    res = fbiomass(x,datmeteo,distribthetaUN[k,])
    res
  })

  t(preds)
}


#' Probabilistic prediction from a distribution of theta and Kriging metamodel
#'
#' @param krig1s Kriging metamodel for a given season
#' @param distribtheta  sample of parameters in the unit cube
#' @return a list with predictions and associated sd for each element of the sample (log scale)
#' @export
#'
#' @examples
#' Bounds  = matrix(c(600,1000,1300,1800,1,6),nrow=3,ncol=2,byrow=TRUE)
#' D = DiceDesign::lhsDesign(30, 3)$design
#' Dun = unNormalise(D,Bounds )
#' met <- BuildMeteodat("2008-2009")
#' nT=10
#' times = seq(160,305,length=nT)
#' y = t(sapply(1:nrow(Dun), function(i) fbiomass(200,met,Dun[i,])[times]))
#' y = log(y)
#' krig1s = lapply(1:ncol(y), function(k){
#' m <- DiceKriging::km(design=D, response=y[,k],nugget = .01,control=list(trace=FALSE))
#' return(m)})
#' dtheta = matrix(runif(300,.4,.45),100,3)
#' PredA = predictApprox(krig1s,dtheta)
#' meanPred = predictmultdim(apply(dtheta,2,mean),krig1s)
#' # Plot mean predicted biomas with Kriging uncertainties
#' plot(met$cumdegree[times],exp(meanPred[1,]),ylab="biomass",ylim=c(0,14))
#' arrows(met$cumdegree[times], exp(meanPred[1,]-1.64*meanPred[2,]),met$cumdegree[times], exp(meanPred[1,]+1.64*meanPred[2,]), length=0.05, angle=90, code=3)
#' # compute quantiles with uncertainties on parameters
#' Simus = sapply(1:ncol(PredA$Mpred),function(k){
#'                           Sim = PredA$Mpred[,k] + rnorm(1000,0,PredA$Spred[,k])})
#' q5 = apply(Simus,2,quantile,.05) ; q95 = apply(Simus,2,quantile,.95)
#' lines(met$cumdegree[times],exp(q5),lty=2) ; lines(met$cumdegree[times],exp(q95),lty=2)
#' legend("topleft",legend=c("mean prediction with Kriging uncertainty","quantiles .05 and .95, full uncertainty"),pch=c(1,NA),lty=c(0,2))
predictApprox=  function(krig1s, distribtheta)
{
  namesX = colnames(krig1s[[1]]@X)
  ntheta = nrow(distribtheta)

  preds = lapply(1:ntheta,function(k){
    predkrig=sapply(krig1s,function(m)
    {
      new = (matrix(distribtheta[k,],nrow=1))
      colnames(new)= namesX
      res = predict(m,newdata=new,type="UK")
      c(res$mean,res$sd)
    })
    return(predkrig)
  })

  Mpred = sapply(preds,function(p) p[1,])
  Spred = sapply(preds,function(p) p[2,])

  return(list(Mpred=t(Mpred),Spred=t(Spred)))

}

