#' Extract meteorological data
#'
#' @param season string of character
#'
#' @return a data.frame with meteorological data including PAR and cumulative degree
#' @export
#'
#' @examples
#' BuildMeteodat("2008-2009")
BuildMeteodat = function(season)
{
  name <- paste0("toycalibr_meteo-data-Climatik_", season, "_processed.tsv")
  path <- system.file("extdata", name, package="EGOcalibrationFSPM")
  dat <- read.table(path, header=TRUE, sep="\t")
  datameteo <- dat[, c("PAR", "TT_basic_cumul")]
  names(datameteo)[2] = "cumdegree"
  return(datameteo)
}



#' Compute the LAI for the toy model
#'
#' @param x cumulative degree
#' @param theta vector of 3 parameters, \code{theta[1]} is linked with the start of the LAI, \code{theta[2]} is linked with the end of t, \code{theta[3]} is linked with the max LAI
#'
#' @return LAI along time
#' @export
#'
#' @examples
#' met <- BuildMeteodat("2008-2009")
#' cumLAI <- computeLAI(met$cumdegree, c(800, 1600, 3))
#' plot(met$cumdegree, cumLAI, type="l")
computeLAI = function(x, theta)
{
  if (length(theta) == 3) {
    theta = c(theta[1:2], 2400, theta[3])
  }
  b1 = (theta[2] - theta[1]) / 10
  b2 = (theta[3] - theta[2]) / 10
  y =  theta[4] / (1 + exp(-(x - theta[1]) / b1))

  y[x > theta[2] - 200] = theta[4] / (1 + exp((x[x > theta[2] - 200] - theta[2]-400) /
                                                b2))

  y
}



#' Compute biomass over time for the toy model
#'
#' @param x sowing density
#' @param datmeteo meteorological data extracted from raw data
#' @param theta parameters tuning the LAI
#'
#' @return cumulative biomass values over time
#' @export
#'
#' @examples
#' met <- BuildMeteodat("2008-2009")
#' cumbiomass <- fbiomass(200, met, c(800, 1600, 3))
#' plot(met$cumdegree, cumbiomass, type="l")
fbiomass = function(x, datmeteo, theta)
{
  # fixed parameters
  epsilon_PAR <- 0.95 # to multiply PAR; from Baey et al (2014)
  RUE <-
    2.8 # 2.8 g per MJ of intercepted PAR or 1.4 g per MJ of solar radiation (Sinclair and Muchow, 1998, p.219)
  k_B <-
    0.59 # median value from table 1 of Liu et al (2021), of the same order of magnitude as 0.45 found in Varlet-Grancher et al (1989) -> Thorne et al (1988)

  # LAI computation
  LAI = computeLAI(datmeteo$cumdegree, theta)
  RIE <- (1 - exp(-k_B * LAI))
  daily_biomass <- (datmeteo$PAR * epsilon_PAR) * (1 / x) * RIE * RUE
  ret = cumsum(daily_biomass)
  ret
}
