## ---- include=FALSE-----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>", fig.width = 8, fig.height = 7
)

## ----setup--------------------------------------------------------------------
suppressPackageStartupMessages(library(EGOcalibrationFSPM))
suppressPackageStartupMessages(library(MCMCpack))
suppressPackageStartupMessages(library(DiceDesign))
suppressPackageStartupMessages(library(DiceKriging))
suppressPackageStartupMessages(library(ggplot2))
suppressPackageStartupMessages(library(cowplot))
suppressPackageStartupMessages(library(ggmcmc))

## ----conf---------------------------------------------------------------------
set.seed(12345)

# sowing density (nb of plants per m2)
x <- 200

# chosen values for the parameters of the FSPM
theta <- c(850, 1600, 3)
ltheta <- length(theta)

# bounds for calibrating theta
Boundstheta = matrix(c( 600, 1000,
                       1300, 1800,
                          1,    6),
                     nrow=3, ncol=2, byrow=TRUE)

# observational noise
sigma <- .02

# times of observation
nobs <- 5
times <- round(seq(160, 305, length=nobs))

# considered seasons
seasons <- list("2019-2020","2018-2019","2017-2018","2016-2017","2015-2016")

## ----setMCMC------------------------------------------------------------------
# cov matrix of the random walk
V = diag(ltheta) * 1e-4
nburnin = 1000
nMCMC = 5e3
npointsEGO = 1000
nsimEGO = 1000

## ----simu---------------------------------------------------------------------
resSimu = lapply(seasons, function(season)
{
  datameteo = BuildMeteodat(season)
  
  ybiom = fbiomass(x, datameteo, theta)
  logybiom = log(ybiom) + rnorm(length(ybiom), 0, sigma)
  logybiom[is.infinite(logybiom)] = -10
  
  return(list(datmeteo=datameteo, logy=logybiom))
})

listdatmeteo = lapply(resSimu, function(res){res$datmeteo})
listlogy = lapply(resSimu, function(res){res$logy[times]})

## ----plot---------------------------------------------------------------------
str(listdatmeteo[[1]])
summary(listdatmeteo[[1]])
summary(exp(listlogy[[1]]))
plot(listdatmeteo[[1]]$cumdegree[times], exp(listlogy[[1]]),
     las=1, type="b", lty=3,
     main="Simulated data", xlab="cumulative growing degree-days",
     ylab="biomass (in g)")

## ----FSPMbayesianCalib--------------------------------------------------------
system.time(
    res1 <- MCMCmetrop1R(exactlogpostMulti, theta.init=rep(.5,ltheta),
                         burnin=nburnin, mcmc=nMCMC, logfun=TRUE, V=V,
                         listlogy=listlogy, x=x, listdatmeteo=listdatmeteo, times=times,
                         sigma2=sigma^2, Bounds=Boundstheta)
)

# reput theta in the correct scale
resUN1 = unNormalise(res1, Boundstheta)
par(mfrow=c(1,3))
for (i in 1:ltheta)
{hist(resUN1[,i], main=paste0("theta[",i,"]"), xlab="", las=1)
  abline(v=theta[i],col=2,lwd=4)}

## ----another chain------------------------------------------------------------
res1b = MCMCmetrop1R(exactlogpostMulti, theta.init=rep(.5,ltheta),
                     burnin=nburnin, mcmc=nMCMC, logfun=TRUE, V=V,
                     listlogy=listlogy, x=x, listdatmeteo=listdatmeteo, times=times,
                     sigma2=sigma^2, Bounds=Boundstheta,seed=1)

## ----bgr----------------------------------------------------------------------
S = ggs(as.mcmc.list(list(res1,res1b)))
ggs_grb(S)

## ----kriging------------------------------------------------------------------
# Exploratory design of experiments
nkrig = 15

# Design of experiments maximin LHS over the dimension of theta
D = lhsDesign(nkrig, ltheta)$design
Dopt = maximinSA_LHS(D, T0=10, c=0.99, it=2000)$design

# unnormalize
Dun = unNormalise(Dopt, Boundstheta)

## ----kriging2-----------------------------------------------------------------
## Running the FSPM only for nkrig runs
YDun = lapply(seasons, function(season)
{
  datameteo = BuildMeteodat(season) 
  
  y = t(sapply(1:nrow(Dun), function(i) fbiomass(x,datameteo,Dun[i,])[times]))
  y = log(y)
  if (any(is.infinite(y))) print("0 value for y")
  y[is.infinite(y)] = -10
  y
})

## ----kriging3-----------------------------------------------------------------
## Fitting kriging metamodels for each time of observation and each meteorological conditions
# setting the nugget
fixedNugget = sigma^2/10
krig = lapply(YDun, function(y)
{
  res=lapply(1:ncol(y), function(k){
    m <- km(design=Dopt, response=y[,k],nugget = fixedNugget,control=list(trace=FALSE)) 
    return(m)
  })
  res
} 
)

## ----loo----------------------------------------------------------------------
X = krig[[1]][[1]]@X
ResListCrossValidationAllmeteo = lapply(krig,function(krigm){
  ResListCrossValidation = lapply(krigm,function(krigmt) {
    Y = krigmt@y
    resloo = sapply(1:nrow(X),function(i){
      Xloo = X[-i,]
      Yloo = Y[-i]
      m <- km(design=Xloo, response=Yloo,nugget = fixedNugget,control=list(trace=FALSE))
      new = (matrix(X[i,],nrow=1))
      colnames(new)= colnames(X)
      pred = predict(m,newdata=new,type="UK")
      return(c(pred$mean,pred$sd,pred$mean-Y[i],(pred$mean-Y[i])/pred$sd))
    }) 
    resloo
  })
  ResListCrossValidation
})


dfCV = data.frame(meteo = rep(paste0("m",1:length(listdatmeteo)),each=nrow(X)*nobs),
                  time = rep(rep(1:nobs,each=nrow(X)),length(listdatmeteo)),
                  Error = numeric(nrow(X)*nobs*length(listdatmeteo)),
                  NormError =   numeric(nrow(X)*nobs*length(listdatmeteo)))
  
cpt=0
for (i in 1:length(listdatmeteo))
{
  for (j in 1:nobs)
  {
    dfCV[cpt+1:nrow(X),3:4] = t(ResListCrossValidationAllmeteo[[i]][[j]][3:4,])
    cpt = cpt + nrow(X)
  }
}

ggplot(dfCV,aes(x=time,y=Error,col=meteo)) +geom_point() +geom_jitter() + ggtitle("Leave-one-out errors")  

ggplot(dfCV,aes(x=time,y=NormError,col=meteo)) +geom_point() +geom_jitter() + ggtitle("Leave-one-out normalized errors")  


## ----calibKrig----------------------------------------------------------------
system.time(
    res2 <- MCMCmetrop1R(approxlogpostMulti, theta.init=rep(.5, ltheta),
                         burnin=nburnin, mcmc=nMCMC, logfun=TRUE, V=V,
                         listlogy=listlogy, krig=krig, sigma2=sigma^2)
)
resUN2 = unNormalise(res2, Boundstheta)
par(mfrow=c(1,3))
for (i in 1:ltheta)
{hist(resUN2[,i], main=paste0("theta[",i,"]"), xlab="", las=1)
  abline(v=theta[i],col=2,lwd=4)}

## ----EGO----------------------------------------------------------------------
nEI = 5
system.time(
    updatedKrig <- runEGO(krig, listlogy, sigma^2, x, seasons, Boundstheta,
                          times, nEI=nEI, npoints=npointsEGO, nsim=nsimEGO)
)

## ----plot new points----------------------------------------------------------
upkrig = updatedKrig$krig
thetaNN = Normalise(theta, Boundstheta)
X = upkrig[[1]][[1]]@X
plot(X[,1:2],col=c(rep(1,nrow(X)-nEI),rep(2,nEI)))
points(x=thetaNN[1],y=thetaNN[2],col=3,pch=20)
plot(X[,c(1,3)],col=c(rep(1,nrow(X)-nEI),rep(2,nEI)))
points(x=thetaNN[1],y=thetaNN[3],col=3,pch=20)

## ----calibKrigEGO-------------------------------------------------------------
system.time(
    res3 <- MCMCmetrop1R(approxlogpostMulti, theta.init=rep(.5, ltheta),
                         burnin=nburnin, mcmc=nMCMC, logfun=TRUE, V=V,
                         listlogy=listlogy, krig=upkrig, sigma2=sigma^2)
)
resUN3 = unNormalise(res3, Boundstheta)
par(mfrow=c(1,3))
for (i in 1:ltheta)
{hist(resUN3[,i], main=paste0("theta[",i,"]"), xlab="", las=1)
  abline(v=theta[i],col=2,lwd=4)}

## ----calib comparison, fig.width=12, fig.height=10----------------------------
lims = matrix(NA, ltheta, 2)
lims[,1] = apply(resUN2, 2, min) - .5
lims[,2] = apply(resUN2, 2, max) + .5

resUN1df = as.data.frame(resUN1)
names(resUN1df) = paste0("theta",1:3)
title <- "True FSPM"
p1 = ggplot(resUN1df,aes(x=theta1))+geom_histogram()+theme_bw() +
  xlim(lims[1,])+ geom_vline(xintercept=theta[1],color=2,size=2) +
  labs(title=title)
p2 = ggplot(resUN1df,aes(x=theta2))+geom_histogram()+theme_bw() +
  xlim(lims[2,])+ geom_vline(xintercept=theta[2],color=2,size=2) +
  labs(title=title)
p3 = ggplot(resUN1df,aes(x=theta3))+geom_histogram()+theme_bw() +
  xlim(lims[3,])+ geom_vline(xintercept=theta[3],color=2,size=2) +
  labs(title=title)

resUN2df = as.data.frame(resUN2)
names(resUN2df) = paste0("theta",1:3)
title <- "Kriging metamodel"
p4 = ggplot(resUN2df,aes(x=theta1))+geom_histogram()+theme_bw() +
  xlim(lims[1,])+ geom_vline(xintercept=theta[1],color=2,size=2) +
  labs(title=title)
p5 = ggplot(resUN2df,aes(x=theta2))+geom_histogram()+theme_bw() +
  xlim(lims[2,])+ geom_vline(xintercept=theta[2],color=2,size=2) +
  labs(title=title)
p6 = ggplot(resUN2df,aes(x=theta3))+geom_histogram()+theme_bw() +
  xlim(lims[3,])+ geom_vline(xintercept=theta[3],color=2,size=2) +
  labs(title=title)

resUN3df = as.data.frame(resUN3)
names(resUN3df) = paste0("theta",1:3)
title <- "EGO-enhanced metamodel"
p7 = ggplot(resUN3df,aes(x=theta1))+geom_histogram()+theme_bw() +
  xlim(lims[1,])+ geom_vline(xintercept=theta[1],color=2,size=2) +
  labs(title=title)
p8 = ggplot(resUN3df,aes(x=theta2))+geom_histogram()+theme_bw() +
  xlim(lims[2,])+ geom_vline(xintercept=theta[2],color=2,size=2) +
  labs(title=title)
p9 = ggplot(resUN3df,aes(x=theta3))+geom_histogram()+theme_bw() +
  xlim(lims[3,])+ geom_vline(xintercept=theta[3],color=2,size=2) +
  labs(title=title)


# enforce the same y-axes

A = ggplot_build(p1)
B = ggplot_build(p4)
C = ggplot_build(p7)
ymax = max(A$layout$panel_scales_y[[1]]$range$range[2],B$layout$panel_scales_y[[1]]$range$range[2],C$layout$panel_scales_y[[1]]$range$range[2])
p1 = p1 + ylim(c(0,ymax))
p4 = p4 + ylim(c(0,ymax))
p7 = p7 + ylim(c(0,ymax))  


A = ggplot_build(p2)
B = ggplot_build(p5)
C = ggplot_build(p8)
ymax = max(A$layout$panel_scales_y[[1]]$range$range[2],B$layout$panel_scales_y[[1]]$range$range[2],C$layout$panel_scales_y[[1]]$range$range[2])
p2 = p2 + ylim(c(0,ymax))
p5 = p5 + ylim(c(0,ymax))
p8 = p8 + ylim(c(0,ymax)) 



A = ggplot_build(p3)
B = ggplot_build(p6)
C = ggplot_build(p9)
ymax = max(A$layout$panel_scales_y[[1]]$range$range[2],B$layout$panel_scales_y[[1]]$range$range[2],C$layout$panel_scales_y[[1]]$range$range[2])
p3 = p3 + ylim(c(0,ymax))
p6 = p6 + ylim(c(0,ymax))
p9 = p9 + ylim(c(0,ymax)) 

plot_grid(p1,p2,p3,p4,p5,p6,p7,p8,p9,nrow=3,ncol=3)



## -----------------------------------------------------------------------------
# true biomass
season = "2011-2012"
datameteo1112 = BuildMeteodat(season)
ybiom1112 = fbiomass(x,datameteo1112,theta)

## -----------------------------------------------------------------------------
Predbiomass = predictExact(x, datameteo1112, res1, Boundstheta)
meanBiom = apply(Predbiomass, 2, mean)
q5 = apply(Predbiomass,2,quantile,.05) ; q95 = apply(Predbiomass,2,quantile,.95)
preddf = data.frame(meanBiomass = meanBiom,
                    cumdegree = datameteo1112$cumdegree,
                    quantile05 = q5, quantile95 = q95)
ggplot(preddf,aes(x=cumdegree,y=meanBiom)) +
  labs(x="cumulative growing degree-days", y="biomass (in g)") +
  geom_line() + ylim(c(0,10)) + theme_bw() +
  geom_ribbon(aes(ymin=quantile05, ymax=quantile95), alpha=.2)

## ----krigingpred--------------------------------------------------------------
# Kriging for the season
y = t(sapply(1:nrow(Dun), function(i) fbiomass(x,datameteo1112,Dun[i,])[times]))
y = log(y)
krig1112=lapply(1:ncol(y), function(k){
  m <- km(design=Dopt, response=y[,k], nugget=fixedNugget,
          control=list(trace=FALSE))
  return(m)
})

# posterior predictions
PredA = predictApprox(krig1112,res2)
meanPred = predictmultdim(apply(res2,2,mean),krig1112)
Simus = sapply(1:ncol(PredA$Mpred),function(k){
  Sim = PredA$Mpred[,k] + rnorm(1000,0,PredA$Spred[,k])})
q5 = apply(Simus,2,quantile,.05) ; q95 = apply(Simus,2,quantile,.95)

predKdf = data.frame(meanBiom = exp(meanPred[1,]),
                     cumdegree = datameteo1112$cumdegree[times],
                     quantile05 = exp(q5), quantile95 = exp(q95))
ggplot(predKdf, aes(cumdegree, meanBiom)) +
  labs(x="cumulative growing degree-days", y="biomass (in g)") +
  geom_point() + geom_line() + ylim(c(0,10)) +
  geom_ribbon(aes(ymin=quantile05, ymax=quantile95), alpha=.2) +
  geom_errorbar(aes(ymin = exp(meanPred[1,] - 1.64*meanPred[2,]),
                    ymax = exp(meanPred[1,] + 1.64*meanPred[2,]))) +
  geom_line(data=preddf, aes(x=cumdegree,y=meanBiom), color="blue") +
  theme_bw()

